@@ -1,14 +1,20 @@
 from optuna.artifacts._backoff import Backoff
 from optuna.artifacts._boto3 import Boto3ArtifactStore
+from optuna.artifacts._download import download_artifact
 from optuna.artifacts._filesystem import FileSystemArtifactStore
 from optuna.artifacts._gcs import GCSArtifactStore
+from optuna.artifacts._list_artifact_meta import get_all_artifact_meta
+from optuna.artifacts._upload import ArtifactMeta
 from optuna.artifacts._upload import upload_artifact
 
 
 __all__ = [
+    "ArtifactMeta",
     "FileSystemArtifactStore",
     "Boto3ArtifactStore",
     "GCSArtifactStore",
     "Backoff",
+    "get_all_artifact_meta",
     "upload_artifact",
+    "download_artifact",
 ]
