@@ -2,17 +2,16 @@
 If you want to add a new command, you also need to update the constant `_COMMANDS`
 """
 
+import argparse
 from argparse import ArgumentParser
 from argparse import Namespace
 import datetime
 from enum import Enum
-from importlib.machinery import SourceFileLoader
 import inspect
 import json
 import logging
 import os
 import sys
-import types
 from typing import Any
 from typing import Dict
 from typing import List
@@ -34,6 +33,8 @@ from optuna.storages import JournalFileStorage
 from optuna.storages import JournalRedisStorage
 from optuna.storages import JournalStorage
 from optuna.storages import RDBStorage
+from optuna.storages.journal import JournalFileBackend
+from optuna.storages.journal import JournalRedisBackend
 from optuna.trial import TrialState
 
 
@@ -60,8 +61,12 @@ def _check_storage_url(storage_url: Optional[str]) -> str:
 def _get_storage(storage_url: Optional[str], storage_class: Optional[str]) -> BaseStorage:
     storage_url = _check_storage_url(storage_url)
     if storage_class:
+        if storage_class == JournalRedisBackend.__name__:
+            return JournalStorage(JournalRedisBackend(storage_url))
         if storage_class == JournalRedisStorage.__name__:
             return JournalStorage(JournalRedisStorage(storage_url))
+        if storage_class == JournalFileBackend.__name__:
+            return JournalStorage(JournalFileBackend(storage_url))
         if storage_class == JournalFileStorage.__name__:
             return JournalStorage(JournalFileStorage(storage_url))
         if storage_class == RDBStorage.__name__:
@@ -69,9 +74,9 @@ def _get_storage(storage_url: Optional[str], storage_class: Optional[str]) -> Ba
         raise CLIUsageError("Unsupported storage class")
 
     if storage_url.startswith("redis"):
-        return JournalStorage(JournalRedisStorage(storage_url))
+        return JournalStorage(JournalRedisBackend(storage_url))
     if os.path.isfile(storage_url):
-        return JournalStorage(JournalFileStorage(storage_url))
+        return JournalStorage(JournalFileBackend(storage_url))
     try:
         return RDBStorage(storage_url)
     except sqlalchemy.exc.ArgumentError:
@@ -268,8 +273,8 @@ def _format_output(
 class _BaseCommand:
     """Base class for commands.
 
-    Note that commands class are not supposed to be called by library users.
-    They are used only in this file to manage optuna CLI commands.
+    Note that command classes are not intended to be called by library users.
+    They are exclusively used within this file to manage Optuna CLI commands.
     """
 
     def __init__(self) -> None:
@@ -364,12 +369,9 @@ class _StudySetUserAttribute(_BaseCommand):
     """Set a user attribute to a study."""
 
     def add_arguments(self, parser: ArgumentParser) -> None:
-        parser.add_argument(
-            "--study", default=None, help="This argument is deprecated. Use --study-name instead."
-        )
         parser.add_argument(
             "--study-name",
-            default=None,
+            required=True,
             help="The name of the study to set the user attribute to.",
         )
         parser.add_argument("--key", "-k", required=True, help="Key of the user attribute.")
@@ -378,19 +380,7 @@ class _StudySetUserAttribute(_BaseCommand):
     def take_action(self, parsed_args: Namespace) -> int:
         storage = _get_storage(parsed_args.storage, parsed_args.storage_class)
 
-        if parsed_args.study and parsed_args.study_name:
-            raise ValueError(
-                "Both `--study-name` and the deprecated `--study` was specified. "
-                "Please remove the `--study` flag."
-            )
-        elif parsed_args.study:
-            message = "The use of `--study` is deprecated. Please use `--study-name` instead."
-            warnings.warn(message, FutureWarning)
-            study = optuna.load_study(storage=storage, study_name=parsed_args.study)
-        elif parsed_args.study_name:
-            study = optuna.load_study(storage=storage, study_name=parsed_args.study_name)
-        else:
-            raise ValueError("Missing study name. Please use `--study-name`.")
+        study = optuna.load_study(storage=storage, study_name=parsed_args.study_name)
 
         study.set_user_attr(parsed_args.key, parsed_args.value)
 
@@ -632,93 +622,6 @@ class _BestTrials(_BaseCommand):
         return 0
 
 
-class _StudyOptimize(_BaseCommand):
-    """Start optimization of a study. Deprecated since version 2.0.0."""
-
-    def add_arguments(self, parser: ArgumentParser) -> None:
-        parser.add_argument(
-            "--n-trials",
-            type=int,
-            help="The number of trials. If this argument is not given, as many "
-            "trials run as possible.",
-        )
-        parser.add_argument(
-            "--timeout",
-            type=float,
-            help="Stop study after the given number of second(s). If this argument"
-            " is not given, as many trials run as possible.",
-        )
-        parser.add_argument(
-            "--n-jobs",
-            type=int,
-            default=1,
-            help="The number of parallel jobs. If this argument is set to -1, the "
-            "number is set to CPU counts.",
-        )
-        parser.add_argument(
-            "--study", default=None, help="This argument is deprecated. Use --study-name instead."
-        )
-        parser.add_argument(
-            "--study-name", default=None, help="The name of the study to start optimization on."
-        )
-        parser.add_argument(
-            "file",
-            help="Python script file where the objective function resides.",
-        )
-        parser.add_argument(
-            "method",
-            help="The method name of the objective function.",
-        )
-
-    def take_action(self, parsed_args: Namespace) -> int:
-        message = (
-            "The use of the `study optimize` command is deprecated. Please execute your Python "
-            "script directly instead."
-        )
-        warnings.warn(message, FutureWarning)
-
-        storage = _get_storage(parsed_args.storage, parsed_args.storage_class)
-
-        if parsed_args.study and parsed_args.study_name:
-            raise ValueError(
-                "Both `--study-name` and the deprecated `--study` was specified. "
-                "Please remove the `--study` flag."
-            )
-        elif parsed_args.study:
-            message = "The use of `--study` is deprecated. Please use `--study-name` instead."
-            warnings.warn(message, FutureWarning)
-            study = optuna.load_study(storage=storage, study_name=parsed_args.study)
-        elif parsed_args.study_name:
-            study = optuna.load_study(storage=storage, study_name=parsed_args.study_name)
-        else:
-            raise ValueError("Missing study name. Please use `--study-name`.")
-
-        # We force enabling the debug flag. As we are going to execute user codes, we want to show
-        # exception stack traces by default.
-        parsed_args.debug = True
-
-        module_name = "optuna_target_module"
-        target_module = types.ModuleType(module_name)
-        loader = SourceFileLoader(module_name, parsed_args.file)
-        loader.exec_module(target_module)
-
-        try:
-            target_method = getattr(target_module, parsed_args.method)
-        except AttributeError:
-            self.logger.error(
-                "Method {} not found in file {}.".format(parsed_args.method, parsed_args.file)
-            )
-            return 1
-
-        study.optimize(
-            target_method,
-            n_trials=parsed_args.n_trials,
-            timeout=parsed_args.timeout,
-            n_jobs=parsed_args.n_jobs,
-        )
-        return 0
-
-
 class _StorageUpgrade(_BaseCommand):
     """Upgrade the schema of an RDB storage."""
 
@@ -754,25 +657,6 @@ class _Ask(_BaseCommand):
 
     def add_arguments(self, parser: ArgumentParser) -> None:
         parser.add_argument("--study-name", type=str, help="Name of study.")
-        parser.add_argument(
-            "--direction",
-            type=str,
-            choices=("minimize", "maximize"),
-            help=(
-                "Direction of optimization. This argument is deprecated."
-                " Please create a study in advance."
-            ),
-        )
-        parser.add_argument(
-            "--directions",
-            type=str,
-            nargs="+",
-            choices=("minimize", "maximize"),
-            help=(
-                "Directions of optimization, if there are multiple objectives."
-                " This argument is deprecated. Please create a study in advance."
-            ),
-        )
         parser.add_argument("--sampler", type=str, help="Class name of sampler object to create.")
         parser.add_argument(
             "--sampler-kwargs",
@@ -813,19 +697,9 @@ class _Ask(_BaseCommand):
         create_study_kwargs = {
             "storage": storage,
             "study_name": parsed_args.study_name,
-            "direction": parsed_args.direction,
-            "directions": parsed_args.directions,
             "load_if_exists": True,
         }
 
-        if parsed_args.direction is not None or parsed_args.directions is not None:
-            message = (
-                "The `direction` and `directions` arguments of the `study ask` command are"
-                " deprecated because the command will no longer create a study when you specify"
-                " the arguments. Please create a study in advance."
-            )
-            warnings.warn(message, FutureWarning)
-
         if parsed_args.sampler is not None:
             if parsed_args.sampler_kwargs is not None:
                 sampler_kwargs = json.loads(parsed_args.sampler_kwargs)
@@ -858,28 +732,12 @@ class _Ask(_BaseCommand):
                 storage=create_study_kwargs["storage"],
                 sampler=create_study_kwargs.get("sampler"),
             )
-            directions = None
-            if (
-                create_study_kwargs["direction"] is not None
-                and create_study_kwargs["directions"] is not None
-            ):
-                raise ValueError("Specify only one of `direction` and `directions`.")
-            if create_study_kwargs["direction"] is not None:
-                directions = [
-                    optuna.study.StudyDirection[create_study_kwargs["direction"].upper()]
-                ]
-            if create_study_kwargs["directions"] is not None:
-                directions = [
-                    optuna.study.StudyDirection[d.upper()]
-                    for d in create_study_kwargs["directions"]
-                ]
-            if directions is not None and study.directions != directions:
-                raise ValueError(
-                    f"Cannot overwrite study direction from {study.directions} to {directions}."
-                )
 
         except KeyError:
-            study = optuna.create_study(**create_study_kwargs)
+            raise KeyError(
+                "Implicit study creation within the 'ask' command was dropped in Optuna v4.0.0. "
+                "Please use the 'create-study' command beforehand."
+            )
         trial = study.ask(fixed_distributions=search_space)
 
         self.logger.info(f"Asked trial {trial.number} with parameters {trial.params}.")
@@ -962,13 +820,29 @@ _COMMANDS: Dict[str, Type[_BaseCommand]] = {
     "trials": _Trials,
     "best-trial": _BestTrial,
     "best-trials": _BestTrials,
-    "study optimize": _StudyOptimize,
     "storage upgrade": _StorageUpgrade,
     "ask": _Ask,
     "tell": _Tell,
 }
 
 
+def _parse_storage_class_without_suggesting_deprecated_choices(value: str) -> str:
+    choices = [
+        RDBStorage.__name__,
+        JournalFileBackend.__name__,
+        JournalRedisBackend.__name__,
+    ]
+    deprecated_choices = [
+        JournalFileStorage.__name__,
+        JournalRedisStorage.__name__,
+    ]
+    if value in choices + deprecated_choices:
+        return value
+    raise argparse.ArgumentTypeError(
+        f"Invalid choice: {value}  (choose from {str(choices)[1:-1]})"
+    )
+
+
 def _add_common_arguments(parser: ArgumentParser) -> ArgumentParser:
     parser.add_argument(
         "--storage",
@@ -980,13 +854,9 @@ def _add_common_arguments(parser: ArgumentParser) -> ArgumentParser:
     )
     parser.add_argument(
         "--storage-class",
-        help="Storage class hint (e.g. JournalFileStorage)",
+        help="Storage class hint (e.g. JournalFileBackend)",
         default=None,
-        choices=[
-            RDBStorage.__name__,
-            JournalFileStorage.__name__,
-            JournalRedisStorage.__name__,
-        ],
+        type=_parse_storage_class_without_suggesting_deprecated_choices,
     )
     verbose_group = parser.add_mutually_exclusive_group()
     verbose_group.add_argument(
@@ -1060,7 +930,7 @@ def _get_parser(description: str = "") -> Tuple[ArgumentParser, Dict[str, Argume
 
 def _preprocess_argv(argv: List[str]) -> List[str]:
     # Some preprocess is necessary for argv because some subcommand includes space
-    # (e.g. optuna study optimize, optuna storage upgrade, ...).
+    # (e.g. optuna storage upgrade).
     argv = argv[1:] if len(argv) > 1 else ["help"]
 
     for i in range(len(argv)):
