@@ -16,13 +16,9 @@ running statistics. Known cases include assessing categorical features with a la
 number of choices since each choice is given a unique one-hot encoded raw feature.
 """
 
-from typing import Dict
-from typing import List
-from typing import Optional
-from typing import Tuple
-from typing import Union
+from __future__ import annotations
 
-import numpy
+import numpy as np
 
 from optuna._imports import try_import
 from optuna.importance._fanova._tree import _FanovaTree
@@ -37,9 +33,9 @@ class _Fanova:
         self,
         n_trees: int,
         max_depth: int,
-        min_samples_split: Union[int, float],
-        min_samples_leaf: Union[int, float],
-        seed: Optional[int],
+        min_samples_split: int | float,
+        min_samples_leaf: int | float,
+        seed: int | None,
     ) -> None:
         _imports.check()
 
@@ -50,16 +46,16 @@ class _Fanova:
             min_samples_leaf=min_samples_leaf,
             random_state=seed,
         )
-        self._trees: Optional[List[_FanovaTree]] = None
-        self._variances: Optional[Dict[int, numpy.ndarray]] = None
-        self._column_to_encoded_columns: Optional[List[numpy.ndarray]] = None
+        self._trees: list[_FanovaTree] | None = None
+        self._variances: dict[int, np.ndarray] | None = None
+        self._column_to_encoded_columns: list[np.ndarray] | None = None
 
     def fit(
         self,
-        X: numpy.ndarray,
-        y: numpy.ndarray,
-        search_spaces: numpy.ndarray,
-        column_to_encoded_columns: List[numpy.ndarray],
+        X: np.ndarray,
+        y: np.ndarray,
+        search_spaces: np.ndarray,
+        column_to_encoded_columns: list[np.ndarray],
     ) -> None:
         assert X.shape[0] == y.shape[0]
         assert X.shape[1] == search_spaces.shape[0]
@@ -76,22 +72,22 @@ class _Fanova:
             # This could occur if for instance `X.shape[0] == 1`.
             raise RuntimeError("Encountered zero total variance in all trees.")
 
-    def get_importance(self, feature: int) -> Tuple[float, float]:
+    def get_importance(self, feature: int) -> tuple[float, float]:
         # Assert that `fit` has been called.
         assert self._trees is not None
         assert self._variances is not None
 
         self._compute_variances(feature)
 
-        fractions: Union[List[float], numpy.ndarray] = []
+        fractions: list[float] | np.ndarray = []
 
         for tree_index, tree in enumerate(self._trees):
             tree_variance = tree.variance
             if tree_variance > 0.0:
                 fraction = self._variances[feature][tree_index] / tree_variance
-                fractions = numpy.append(fractions, fraction)
+                fractions = np.append(fractions, fraction)
 
-        fractions = numpy.asarray(fractions)
+        fractions = np.asarray(fractions)
 
         return float(fractions.mean()), float(fractions.std())
 
@@ -104,9 +100,9 @@ class _Fanova:
             return
 
         raw_features = self._column_to_encoded_columns[feature]
-        variances = numpy.empty(len(self._trees), dtype=numpy.float64)
+        variances = np.empty(len(self._trees), dtype=np.float64)
 
         for tree_index, tree in enumerate(self._trees):
             marginal_variance = tree.get_marginal_variance(raw_features)
-            variances[tree_index] = numpy.clip(marginal_variance, 0.0, None)
+            variances[tree_index] = np.clip(marginal_variance, 0.0, None)
         self._variances[feature] = variances
