@@ -4,19 +4,28 @@ from types import ModuleType
 from typing import Any
 from typing import TYPE_CHECKING
 
-import optuna_integration.lightgbm as lgb
+from optuna._imports import _INTEGRATION_IMPORT_ERROR_TEMPLATE
+
+
+try:
+    import optuna_integration.lightgbm as lgb
+except ModuleNotFoundError:
+    raise ModuleNotFoundError(_INTEGRATION_IMPORT_ERROR_TEMPLATE.format("lightgbm"))
 
 
 if TYPE_CHECKING:
+    # These modules are from optuna-integration.
     from optuna.integration.lightgbm_tuner import LightGBMPruningCallback
     from optuna.integration.lightgbm_tuner import LightGBMTuner
     from optuna.integration.lightgbm_tuner import LightGBMTunerCV
+    from optuna.integration.lightgbm_tuner import train
 
 
 __all__ = [
     "LightGBMPruningCallback",
     "LightGBMTuner",
     "LightGBMTunerCV",
+    "train",
 ]
 
 
