@@ -1,9 +1,8 @@
+from __future__ import annotations
+
 import binascii
+from collections.abc import Container
 import math
-from typing import Container
-from typing import List
-from typing import Optional
-from typing import Union
 
 import optuna
 from optuna import logging
@@ -22,7 +21,7 @@ class HyperbandPruner(BasePruner):
     :math:`n` as its hyperparameter.  For a given finite budget :math:`B`,
     all the configurations have the resources of :math:`B \\over n` on average.
     As you can see, there will be a trade-off of :math:`B` and :math:`B \\over n`.
-    `Hyperband <http://www.jmlr.org/papers/volume18/16-558/16-558.pdf>`_ attacks this trade-off
+    `Hyperband <http://www.jmlr.org/papers/volume18/16-558/16-558.pdf>`__ attacks this trade-off
     by trying different :math:`n` values for a fixed budget.
 
     .. note::
@@ -30,7 +29,7 @@ class HyperbandPruner(BasePruner):
           is used.
         * Optuna uses :class:`~optuna.samplers.TPESampler` by default.
         * `The benchmark result
-          <https://github.com/optuna/optuna/pull/828#issuecomment-575457360>`_
+          <https://github.com/optuna/optuna/pull/828#issuecomment-575457360>`__
           shows that :class:`optuna.pruners.HyperbandPruner` supports both samplers.
 
     .. note::
@@ -59,7 +58,7 @@ class HyperbandPruner(BasePruner):
         (\\frac{\\texttt{max}\\_\\texttt{resource}}{\\texttt{min}\\_\\texttt{resource}})) + 1`.
         Please set ``reduction_factor`` so that the number of brackets is not too large (about 4 –
         6 in most use cases). Please see Section 3.6 of the `original paper
-        <http://www.jmlr.org/papers/volume18/16-558/16-558.pdf>`_ for the detail.
+        <http://www.jmlr.org/papers/volume18/16-558/16-558.pdf>`__ for the detail.
 
     Example:
 
@@ -139,18 +138,18 @@ class HyperbandPruner(BasePruner):
     def __init__(
         self,
         min_resource: int = 1,
-        max_resource: Union[str, int] = "auto",
+        max_resource: str | int = "auto",
         reduction_factor: int = 3,
         bootstrap_count: int = 0,
     ) -> None:
         self._min_resource = min_resource
         self._max_resource = max_resource
         self._reduction_factor = reduction_factor
-        self._pruners: List[SuccessiveHalvingPruner] = []
+        self._pruners: list[SuccessiveHalvingPruner] = []
         self._bootstrap_count = bootstrap_count
         self._total_trial_allocation_budget = 0
-        self._trial_allocation_budgets: List[int] = []
-        self._n_brackets: Optional[int] = None
+        self._trial_allocation_budgets: list[int] = []
+        self._n_brackets: int | None = None
 
         if not isinstance(self._max_resource, int) and self._max_resource != "auto":
             raise ValueError(
@@ -236,7 +235,7 @@ class HyperbandPruner(BasePruner):
         """Compute the index of bracket for a trial of ``trial_number``.
 
         The index of a bracket is noted as :math:`s` in
-        `Hyperband paper <http://www.jmlr.org/papers/volume18/16-558/16-558.pdf>`_.
+        `Hyperband paper <http://www.jmlr.org/papers/volume18/16-558/16-558.pdf>`__.
         """
 
         if len(self._pruners) == 0:
@@ -296,8 +295,8 @@ class HyperbandPruner(BasePruner):
             def get_trials(
                 self,
                 deepcopy: bool = True,
-                states: Optional[Container[TrialState]] = None,
-            ) -> List["optuna.trial.FrozenTrial"]:
+                states: Container[TrialState] | None = None,
+            ) -> list["optuna.trial.FrozenTrial"]:
                 trials = super()._get_trials(deepcopy=deepcopy, states=states)
                 pruner = self.pruner
                 assert isinstance(pruner, HyperbandPruner)
