@@ -1,19 +1,23 @@
+from __future__ import annotations
+
 import abc
+from collections.abc import Callable
+from collections.abc import Sequence
 from typing import Any
-from typing import Callable
-from typing import Dict
-from typing import Optional
-from typing import Sequence
+from typing import TYPE_CHECKING
 import warnings
 
 import numpy as np
 
 from optuna.distributions import BaseDistribution
-from optuna.study import Study
 from optuna.trial import FrozenTrial
 from optuna.trial import TrialState
 
 
+if TYPE_CHECKING:
+    from optuna.study import Study
+
+
 class BaseSampler(abc.ABC):
     """Base class for samplers.
 
@@ -53,7 +57,7 @@ class BaseSampler(abc.ABC):
     @abc.abstractmethod
     def infer_relative_search_space(
         self, study: Study, trial: FrozenTrial
-    ) -> Dict[str, BaseDistribution]:
+    ) -> dict[str, BaseDistribution]:
         """Infer the search space that will be used by relative sampling in the target trial.
 
         This method is called right before :func:`~optuna.samplers.BaseSampler.sample_relative`
@@ -80,8 +84,8 @@ class BaseSampler(abc.ABC):
 
     @abc.abstractmethod
     def sample_relative(
-        self, study: Study, trial: FrozenTrial, search_space: Dict[str, BaseDistribution]
-    ) -> Dict[str, Any]:
+        self, study: Study, trial: FrozenTrial, search_space: dict[str, BaseDistribution]
+    ) -> dict[str, Any]:
         """Sample parameters in a given search space.
 
         This method is called once at the beginning of each trial, i.e., right before the
@@ -175,7 +179,7 @@ class BaseSampler(abc.ABC):
         study: Study,
         trial: FrozenTrial,
         state: TrialState,
-        values: Optional[Sequence[float]],
+        values: Sequence[float] | None,
     ) -> None:
         """Trial post-processing.
 
