@@ -5,6 +5,7 @@ from collections.abc import Callable
 from collections.abc import Sequence
 import hashlib
 from typing import Any
+from typing import TYPE_CHECKING
 
 import numpy as np
 
@@ -22,11 +23,14 @@ from optuna.samplers.nsgaii._child_generation_strategy import NSGAIIChildGenerat
 from optuna.samplers.nsgaii._crossovers._base import BaseCrossover
 from optuna.samplers.nsgaii._crossovers._uniform import UniformCrossover
 from optuna.search_space import IntersectionSearchSpace
-from optuna.study import Study
 from optuna.trial import FrozenTrial
 from optuna.trial import TrialState
 
 
+if TYPE_CHECKING:
+    from optuna.study import Study
+
+
 # Define key names of `Trial.system_attrs`.
 _GENERATION_KEY = "nsga3:generation"
 _POPULATION_CACHE_KEY_PREFIX = "nsga3:population"
@@ -43,11 +47,11 @@ class NSGAIIISampler(BaseSampler):
 
     - `An Evolutionary Many-Objective Optimization Algorithm Using Reference-Point-Based
       Nondominated Sorting Approach, Part I: Solving Problems With Box Constraints
-      <https://doi.org/10.1109/TEVC.2013.2281535>`_
+      <https://doi.org/10.1109/TEVC.2013.2281535>`__
     - `An Evolutionary Many-Objective Optimization Algorithm Using Reference-Point-Based
       Nondominated Sorting Approach, Part II: Handling Constraints and Extending to an Adaptive
       Approach
-      <https://doi.org/10.1109/TEVC.2013.2281534>`_
+      <https://doi.org/10.1109/TEVC.2013.2281534>`__
 
     Args:
         reference_points:
