@@ -1,21 +1,20 @@
 from __future__ import annotations
 
+from collections.abc import Callable
+from collections.abc import Sequence
 import math
 from typing import Any
-from typing import Callable
 from typing import cast
-from typing import Dict
-from typing import Optional
-from typing import Sequence
+from typing import TYPE_CHECKING
 import warnings
 
 import numpy as np
 
-from optuna._hypervolume import WFG
+from optuna._experimental import warn_experimental_argument
+from optuna._hypervolume import compute_hypervolume
 from optuna._hypervolume.hssp import _solve_hssp
 from optuna.distributions import BaseDistribution
 from optuna.distributions import CategoricalChoiceType
-from optuna.exceptions import ExperimentalWarning
 from optuna.logging import get_logger
 from optuna.samplers._base import _CONSTRAINTS_KEY
 from optuna.samplers._base import _process_constraints_after_trial
@@ -27,13 +26,16 @@ from optuna.samplers._tpe.parzen_estimator import _ParzenEstimatorParameters
 from optuna.search_space import IntersectionSearchSpace
 from optuna.search_space.group_decomposed import _GroupDecomposedSearchSpace
 from optuna.search_space.group_decomposed import _SearchSpaceGroup
-from optuna.study import Study
-from optuna.study._multi_objective import _fast_non_dominated_sort
+from optuna.study._multi_objective import _fast_non_domination_rank
 from optuna.study._study_direction import StudyDirection
 from optuna.trial import FrozenTrial
 from optuna.trial import TrialState
 
 
+if TYPE_CHECKING:
+    from optuna.study import Study
+
+
 EPS = 1e-12
 _logger = get_logger(__name__)
 
@@ -60,9 +62,6 @@ def default_weights(x: int) -> np.ndarray:
 class TPESampler(BaseSampler):
     """Sampler using TPE (Tree-structured Parzen Estimator) algorithm.
 
-    This sampler is based on *independent sampling*.
-    See also :class:`~optuna.samplers.BaseSampler` for more details of 'independent sampling'.
-
     On each trial, for each parameter, TPE fits one Gaussian Mixture Model (GMM) ``l(x)`` to
     the set of parameter values associated with the best objective values, and another GMM
     ``g(x)`` to the remaining parameter values. It chooses the parameter value ``x`` that
@@ -71,17 +70,24 @@ class TPESampler(BaseSampler):
     For further information about TPE algorithm, please refer to the following papers:
 
     - `Algorithms for Hyper-Parameter Optimization
-      <https://papers.nips.cc/paper/4443-algorithms-for-hyper-parameter-optimization.pdf>`_
+      <https://papers.nips.cc/paper/4443-algorithms-for-hyper-parameter-optimization.pdf>`__
     - `Making a Science of Model Search: Hyperparameter Optimization in Hundreds of
-      Dimensions for Vision Architectures <http://proceedings.mlr.press/v28/bergstra13.pdf>`_
+      Dimensions for Vision Architectures <http://proceedings.mlr.press/v28/bergstra13.pdf>`__
     - `Tree-Structured Parzen Estimator: Understanding Its Algorithm Components and Their Roles for
-      Better Empirical Performance <https://arxiv.org/abs/2304.11127>`_
+      Better Empirical Performance <https://arxiv.org/abs/2304.11127>`__
 
     For multi-objective TPE (MOTPE), please refer to the following papers:
 
     - `Multiobjective Tree-Structured Parzen Estimator for Computationally Expensive Optimization
-      Problems <https://doi.org/10.1145/3377930.3389817>`_
-    - `Multiobjective Tree-Structured Parzen Estimator <https://doi.org/10.1613/jair.1.13188>`_
+      Problems <https://doi.org/10.1145/3377930.3389817>`__
+    - `Multiobjective Tree-Structured Parzen Estimator <https://doi.org/10.1613/jair.1.13188>`__
+
+    Please also check our articles:
+
+    - `Significant Speed Up of Multi-Objective TPESampler in Optuna v4.0.0
+      <https://medium.com/optuna/significant-speed-up-of-multi-objective-tpesampler-in-optuna-v4-0-0-2bacdcd1d99b>`__
+    - `Multivariate TPE Makes Optuna Even More Powerful
+      <https://medium.com/optuna/multivariate-tpe-makes-optuna-even-more-powerful-63c4bfbaebe2>`__
 
     Example:
         An example of a single-objective optimization is as follows:
@@ -101,26 +107,13 @@ class TPESampler(BaseSampler):
             study.optimize(objective, n_trials=10)
 
     .. note::
-        :class:`~optuna.samplers.TPESampler` can handle a multi-objective task as well and
-        the following shows an example:
-
-        .. testcode::
-
-            import optuna
-
-
-            def objective(trial):
-                x = trial.suggest_float("x", -100, 100)
-                y = trial.suggest_categorical("y", [-1, 0, 1])
-                f1 = x**2 + y
-                f2 = -((x - 2) ** 2 + y)
-                return f1, f2
-
-
-            # We minimize the first objective and maximize the second objective.
-            sampler = optuna.samplers.TPESampler()
-            study = optuna.create_study(directions=["minimize", "maximize"], sampler=sampler)
-            study.optimize(objective, n_trials=100)
+        :class:`~optuna.samplers.TPESampler`, which became much faster in v4.0.0, c.f. `our article
+        <https://medium.com/optuna/significant-speed-up-of-multi-objective-tpesampler-in-optuna-v4-0-0-2bacdcd1d99b>`__,
+        can handle multi-objective optimization with many trials as well.
+        Please note that :class:`~optuna.samplers.NSGAIISampler` will be used by default for
+        multi-objective optimization, so if users would like to use
+        :class:`~optuna.samplers.TPESampler` for multi-objective optimization, ``sampler`` must be
+        explicitly specified when study is created.
 
     Args:
         consider_prior:
@@ -152,23 +145,25 @@ class TPESampler(BaseSampler):
         weights:
             A function that takes the number of finished trials and returns a weight for them.
             See `Making a Science of Model Search: Hyperparameter Optimization in Hundreds of
-            Dimensions for Vision Architectures <http://proceedings.mlr.press/v28/bergstra13.pdf>`_
-            for more details.
+            Dimensions for Vision Architectures
+            <http://proceedings.mlr.press/v28/bergstra13.pdf>`__ for more details.
 
             .. note::
                 In the multi-objective case, this argument is only used to compute the weights of
                 bad trials, i.e., trials to construct `g(x)` in the `paper
-                <https://papers.nips.cc/paper/4443-algorithms-for-hyper-parameter-optimization.pdf>`_
+                <https://papers.nips.cc/paper/4443-algorithms-for-hyper-parameter-optimization.pdf>`__
                 ). The weights of good trials, i.e., trials to construct `l(x)`, are computed by a
                 rule based on the hypervolume contribution proposed in the `paper of MOTPE
-                <https://doi.org/10.1613/jair.1.13188>`_.
+                <https://doi.org/10.1613/jair.1.13188>`__.
         seed:
             Seed for random number generator.
         multivariate:
             If this is :obj:`True`, the multivariate TPE is used when suggesting parameters.
             The multivariate TPE is reported to outperform the independent TPE. See `BOHB: Robust
             and Efficient Hyperparameter Optimization at Scale
-            <http://proceedings.mlr.press/v80/falkner18a.html>`_ for more details.
+            <http://proceedings.mlr.press/v80/falkner18a.html>`__ and `our article
+            <https://medium.com/optuna/multivariate-tpe-makes-optuna-even-more-powerful-63c4bfbaebe2>`__
+            for more details.
 
             .. note::
                 Added in v2.2.0 as an experimental feature. The interface may change in newer
@@ -279,16 +274,16 @@ class TPESampler(BaseSampler):
         n_ei_candidates: int = 24,
         gamma: Callable[[int], int] = default_gamma,
         weights: Callable[[int], np.ndarray] = default_weights,
-        seed: Optional[int] = None,
+        seed: int | None = None,
         *,
         multivariate: bool = False,
         group: bool = False,
         warn_independent_sampling: bool = True,
         constant_liar: bool = False,
-        constraints_func: Optional[Callable[[FrozenTrial], Sequence[float]]] = None,
-        categorical_distance_func: Optional[
-            dict[str, Callable[[CategoricalChoiceType, CategoricalChoiceType], float]]
-        ] = None,
+        constraints_func: Callable[[FrozenTrial], Sequence[float]] | None = None,
+        categorical_distance_func: (
+            dict[str, Callable[[CategoricalChoiceType, CategoricalChoiceType], float]] | None
+        ) = None,
     ) -> None:
         self._parzen_estimator_parameters = _ParzenEstimatorParameters(
             consider_prior,
@@ -309,51 +304,33 @@ class TPESampler(BaseSampler):
 
         self._multivariate = multivariate
         self._group = group
-        self._group_decomposed_search_space: Optional[_GroupDecomposedSearchSpace] = None
-        self._search_space_group: Optional[_SearchSpaceGroup] = None
+        self._group_decomposed_search_space: _GroupDecomposedSearchSpace | None = None
+        self._search_space_group: _SearchSpaceGroup | None = None
         self._search_space = IntersectionSearchSpace(include_pruned=True)
         self._constant_liar = constant_liar
         self._constraints_func = constraints_func
+        # NOTE(nabenabe0928): Users can overwrite _ParzenEstimator to customize the TPE behavior.
+        self._parzen_estimator_cls = _ParzenEstimator
 
         if multivariate:
-            warnings.warn(
-                "``multivariate`` option is an experimental feature."
-                " The interface can change in the future.",
-                ExperimentalWarning,
-            )
+            warn_experimental_argument("multivariate")
 
         if group:
             if not multivariate:
                 raise ValueError(
                     "``group`` option can only be enabled when ``multivariate`` is enabled."
                 )
-            warnings.warn(
-                "``group`` option is an experimental feature."
-                " The interface can change in the future.",
-                ExperimentalWarning,
-            )
+            warn_experimental_argument("group")
             self._group_decomposed_search_space = _GroupDecomposedSearchSpace(True)
 
         if constant_liar:
-            warnings.warn(
-                "``constant_liar`` option is an experimental feature."
-                " The interface can change in the future.",
-                ExperimentalWarning,
-            )
+            warn_experimental_argument("constant_liar")
 
         if constraints_func is not None:
-            warnings.warn(
-                "The ``constraints_func`` option is an experimental feature."
-                " The interface can change in the future.",
-                ExperimentalWarning,
-            )
+            warn_experimental_argument("constraints_func")
 
         if categorical_distance_func is not None:
-            warnings.warn(
-                "The ``categorical_distance_func`` option is an experimental feature."
-                " The interface can change in the future.",
-                ExperimentalWarning,
-            )
+            warn_experimental_argument("categorical_distance_func")
 
     def reseed_rng(self) -> None:
         self._rng.rng.seed()
@@ -361,11 +338,11 @@ class TPESampler(BaseSampler):
 
     def infer_relative_search_space(
         self, study: Study, trial: FrozenTrial
-    ) -> Dict[str, BaseDistribution]:
+    ) -> dict[str, BaseDistribution]:
         if not self._multivariate:
             return {}
 
-        search_space: Dict[str, BaseDistribution] = {}
+        search_space: dict[str, BaseDistribution] = {}
 
         if self._group:
             assert self._group_decomposed_search_space is not None
@@ -386,8 +363,8 @@ class TPESampler(BaseSampler):
         return search_space
 
     def sample_relative(
-        self, study: Study, trial: FrozenTrial, search_space: Dict[str, BaseDistribution]
-    ) -> Dict[str, Any]:
+        self, study: Study, trial: FrozenTrial, search_space: dict[str, BaseDistribution]
+    ) -> dict[str, Any]:
         if self._group:
             assert self._search_space_group is not None
             params = {}
@@ -403,8 +380,8 @@ class TPESampler(BaseSampler):
             return self._sample_relative(study, trial, search_space)
 
     def _sample_relative(
-        self, study: Study, trial: FrozenTrial, search_space: Dict[str, BaseDistribution]
-    ) -> Dict[str, Any]:
+        self, study: Study, trial: FrozenTrial, search_space: dict[str, BaseDistribution]
+    ) -> dict[str, Any]:
         if search_space == {}:
             return {}
 
@@ -459,8 +436,8 @@ class TPESampler(BaseSampler):
         return {k: np.asarray(v) for k, v in values.items()}
 
     def _sample(
-        self, study: Study, trial: FrozenTrial, search_space: Dict[str, BaseDistribution]
-    ) -> Dict[str, Any]:
+        self, study: Study, trial: FrozenTrial, search_space: dict[str, BaseDistribution]
+    ) -> dict[str, Any]:
         if self._constant_liar:
             states = [TrialState.COMPLETE, TrialState.PRUNED, TrialState.RUNNING]
         else:
@@ -510,11 +487,17 @@ class TPESampler(BaseSampler):
             weights_below = _calculate_weights_below_for_multi_objective(
                 study, trials, self._constraints_func
             )[param_mask_below]
-            mpe = _ParzenEstimator(
+            assert np.isfinite(weights_below).all()
+            mpe = self._parzen_estimator_cls(
                 observations, search_space, self._parzen_estimator_parameters, weights_below
             )
         else:
-            mpe = _ParzenEstimator(observations, search_space, self._parzen_estimator_parameters)
+            mpe = self._parzen_estimator_cls(
+                observations, search_space, self._parzen_estimator_parameters
+            )
+
+        if not isinstance(mpe, _ParzenEstimator):
+            raise RuntimeError("_parzen_estimator_cls must override _ParzenEstimator.")
 
         return mpe
 
@@ -531,9 +514,7 @@ class TPESampler(BaseSampler):
 
     @classmethod
     def _compare(
-        cls,
-        samples: Dict[str, np.ndarray],
-        acquisition_func_vals: np.ndarray,
+        cls, samples: dict[str, np.ndarray], acquisition_func_vals: np.ndarray
     ) -> dict[str, int | float]:
         sample_size = next(iter(samples.values())).size
         if sample_size == 0:
@@ -550,7 +531,7 @@ class TPESampler(BaseSampler):
         return {k: v[best_idx].item() for k, v in samples.items()}
 
     @staticmethod
-    def hyperopt_parameters() -> Dict[str, Any]:
+    def hyperopt_parameters() -> dict[str, Any]:
         """Return the the default parameters of hyperopt (v0.1.2).
 
         :class:`~optuna.samplers.TPESampler` can be instantiated with the parameters returned
@@ -559,7 +540,7 @@ class TPESampler(BaseSampler):
         Example:
 
             Create a :class:`~optuna.samplers.TPESampler` instance with the default
-            parameters of `hyperopt <https://github.com/hyperopt/hyperopt/tree/0.1.2>`_.
+            parameters of `hyperopt <https://github.com/hyperopt/hyperopt/tree/0.1.2>`__.
 
             .. testcode::
 
@@ -600,7 +581,7 @@ class TPESampler(BaseSampler):
         study: Study,
         trial: FrozenTrial,
         state: TrialState,
-        values: Optional[Sequence[float]],
+        values: Sequence[float] | None,
     ) -> None:
         assert state in [TrialState.COMPLETE, TrialState.FAIL, TrialState.PRUNED]
         if self._constraints_func is not None:
@@ -609,10 +590,7 @@ class TPESampler(BaseSampler):
 
 
 def _split_trials(
-    study: Study,
-    trials: list[FrozenTrial],
-    n_below: int,
-    constraints_enabled: bool,
+    study: Study, trials: list[FrozenTrial], n_below: int, constraints_enabled: bool
 ) -> tuple[list[FrozenTrial], list[FrozenTrial]]:
     complete_trials = []
     pruned_trials = []
@@ -661,9 +639,7 @@ def _split_complete_trials(
 
 
 def _split_complete_trials_single_objective(
-    trials: Sequence[FrozenTrial],
-    study: Study,
-    n_below: int,
+    trials: Sequence[FrozenTrial], study: Study, n_below: int
 ) -> tuple[list[FrozenTrial], list[FrozenTrial]]:
     if study.direction == StudyDirection.MINIMIZE:
         sorted_trials = sorted(trials, key=lambda trial: cast(float, trial.value))
@@ -673,9 +649,7 @@ def _split_complete_trials_single_objective(
 
 
 def _split_complete_trials_multi_objective(
-    trials: Sequence[FrozenTrial],
-    study: Study,
-    n_below: int,
+    trials: Sequence[FrozenTrial], study: Study, n_below: int
 ) -> tuple[list[FrozenTrial], list[FrozenTrial]]:
     if n_below == 0:
         # The type of trials must be `list`, but not `Sequence`.
@@ -685,7 +659,7 @@ def _split_complete_trials_multi_objective(
     lvals *= np.array([-1.0 if d == StudyDirection.MAXIMIZE else 1.0 for d in study.directions])
 
     # Solving HSSP for variables number of times is a waste of time.
-    nondomination_ranks = _fast_non_dominated_sort(lvals, n_below=n_below)
+    nondomination_ranks = _fast_non_domination_rank(lvals, n_below=n_below)
     assert 0 <= n_below <= len(lvals)
 
     indices = np.array(range(len(lvals)))
@@ -735,9 +709,7 @@ def _get_pruned_trial_score(trial: FrozenTrial, study: Study) -> tuple[float, fl
 
 
 def _split_pruned_trials(
-    trials: Sequence[FrozenTrial],
-    study: Study,
-    n_below: int,
+    trials: Sequence[FrozenTrial], study: Study, n_below: int
 ) -> tuple[list[FrozenTrial], list[FrozenTrial]]:
     n_below = min(n_below, len(trials))
     sorted_trials = sorted(trials, key=lambda trial: _get_pruned_trial_score(trial, study))
@@ -798,13 +770,22 @@ def _calculate_weights_below_for_multi_objective(
         worst_point = np.max(lvals, axis=0)
         reference_point = np.maximum(1.1 * worst_point, 0.9 * worst_point)
         reference_point[reference_point == 0] = EPS
-        hv = WFG().compute(lvals, reference_point)
+        hv = compute_hypervolume(lvals, reference_point)
         indices_mat = ~np.eye(n_below).astype(bool)
         contributions = np.asarray(
-            [hv - WFG().compute(lvals[indices_mat[i]], reference_point) for i in range(n_below)]
+            [
+                hv - compute_hypervolume(lvals[indices_mat[i]], reference_point)
+                for i in range(n_below)
+            ]
         )
-        contributions += EPS
-        weights_below = np.clip(contributions / np.max(contributions), 0, 1)
+        contributions[np.isnan(contributions)] = np.inf
+        max_contribution = np.maximum(np.max(contributions), EPS)
+        if not np.isfinite(max_contribution):
+            weights_below = np.ones_like(contributions, dtype=float)
+            # TODO(nabenabe0928): Make the weights for non Pareto solutions to zero.
+            weights_below[np.isfinite(contributions)] = EPS
+        else:
+            weights_below = np.clip(contributions / max_contribution, EPS, 1)
 
     # For now, EPS weight is assigned to infeasible trials.
     weights_below_all = np.full(len(below_trials), EPS)
