@@ -3,10 +3,14 @@ from __future__ import annotations
 import copy
 from typing import Dict
 from typing import Tuple
+from typing import TYPE_CHECKING
 
 import optuna
 from optuna.distributions import BaseDistribution
-from optuna.study import Study
+
+
+if TYPE_CHECKING:
+    from optuna.study import Study
 
 
 def _calculate(
@@ -92,8 +96,10 @@ class IntersectionSearchSpace:
         if self._study_id is None:
             self._study_id = study._study_id
         else:
-            # Note that the check below is meaningless when `InMemoryStorage` is used
-            # because `InMemoryStorage.create_new_study` always returns the same study ID.
+            # Note that the check below is meaningless when
+            # :class:`~optuna.storages.InMemoryStorage` is used because
+            # :func:`~optuna.storages.InMemoryStorage.create_new_study`
+            # always returns the same study ID.
             if self._study_id != study._study_id:
                 raise ValueError("`IntersectionSearchSpace` cannot handle multiple studies.")
 
