@@ -1,13 +1,10 @@
+from __future__ import annotations
+
 import abc
+from collections.abc import Container
+from collections.abc import Sequence
 from typing import Any
 from typing import cast
-from typing import Container
-from typing import Dict
-from typing import List
-from typing import Optional
-from typing import Sequence
-from typing import Tuple
-from typing import Union
 
 from optuna._typing import JSONSerializable
 from optuna.distributions import BaseDistribution
@@ -52,7 +49,7 @@ class BaseStorage(abc.ABC):
 
     @abc.abstractmethod
     def create_new_study(
-        self, directions: Sequence[StudyDirection], study_name: Optional[str] = None
+        self, directions: Sequence[StudyDirection], study_name: str | None = None
     ) -> int:
         """Create a new study from a name.
 
@@ -74,7 +71,6 @@ class BaseStorage(abc.ABC):
             :exc:`optuna.exceptions.DuplicatedStudyError`:
                 If a study with the same ``study_name`` already exists.
         """
-        # TODO(ytsmiling) Fix RDB storage implementation to ensure unique `study_id`.
         raise NotImplementedError
 
     @abc.abstractmethod
@@ -168,7 +164,7 @@ class BaseStorage(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def get_study_directions(self, study_id: int) -> List[StudyDirection]:
+    def get_study_directions(self, study_id: int) -> list[StudyDirection]:
         """Read whether a study maximizes or minimizes an objective.
 
         Args:
@@ -185,7 +181,7 @@ class BaseStorage(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def get_study_user_attrs(self, study_id: int) -> Dict[str, Any]:
+    def get_study_user_attrs(self, study_id: int) -> dict[str, Any]:
         """Read the user-defined attributes of a study.
 
         Args:
@@ -202,7 +198,7 @@ class BaseStorage(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def get_study_system_attrs(self, study_id: int) -> Dict[str, Any]:
+    def get_study_system_attrs(self, study_id: int) -> dict[str, Any]:
         """Read the optuna-internal attributes of a study.
 
         Args:
@@ -219,7 +215,7 @@ class BaseStorage(abc.ABC):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def get_all_studies(self) -> List[FrozenStudy]:
+    def get_all_studies(self) -> list[FrozenStudy]:
         """Read a list of :class:`~optuna.study.FrozenStudy` objects.
 
         Returns:
@@ -231,7 +227,7 @@ class BaseStorage(abc.ABC):
     # Basic trial manipulation
 
     @abc.abstractmethod
-    def create_new_trial(self, study_id: int, template_trial: Optional[FrozenTrial] = None) -> int:
+    def create_new_trial(self, study_id: int, template_trial: FrozenTrial | None = None) -> int:
         """Create and add a new trial to a study.
 
         The returned trial ID is unique among all current and deleted trials.
@@ -347,7 +343,7 @@ class BaseStorage(abc.ABC):
 
     @abc.abstractmethod
     def set_trial_state_values(
-        self, trial_id: int, state: TrialState, values: Optional[Sequence[float]] = None
+        self, trial_id: int, state: TrialState, values: Sequence[float] | None = None
     ) -> bool:
         """Update the state and values of a trial.
 
@@ -469,8 +465,8 @@ class BaseStorage(abc.ABC):
         self,
         study_id: int,
         deepcopy: bool = True,
-        states: Optional[Container[TrialState]] = None,
-    ) -> List[FrozenTrial]:
+        states: Container[TrialState] | None = None,
+    ) -> list[FrozenTrial]:
         """Read all trials in a study.
 
         Args:
@@ -492,7 +488,7 @@ class BaseStorage(abc.ABC):
         raise NotImplementedError
 
     def get_n_trials(
-        self, study_id: int, state: Optional[Union[Tuple[TrialState, ...], TrialState]] = None
+        self, study_id: int, state: tuple[TrialState, ...] | TrialState | None = None
     ) -> int:
         """Count the number of trials in a study.
 
@@ -554,7 +550,7 @@ class BaseStorage(abc.ABC):
 
         return best_trial
 
-    def get_trial_params(self, trial_id: int) -> Dict[str, Any]:
+    def get_trial_params(self, trial_id: int) -> dict[str, Any]:
         """Read the parameter dictionary of a trial.
 
         Args:
@@ -571,7 +567,7 @@ class BaseStorage(abc.ABC):
         """
         return self.get_trial(trial_id).params
 
-    def get_trial_user_attrs(self, trial_id: int) -> Dict[str, Any]:
+    def get_trial_user_attrs(self, trial_id: int) -> dict[str, Any]:
         """Read the user-defined attributes of a trial.
 
         Args:
@@ -587,7 +583,7 @@ class BaseStorage(abc.ABC):
         """
         return self.get_trial(trial_id).user_attrs
 
-    def get_trial_system_attrs(self, trial_id: int) -> Dict[str, Any]:
+    def get_trial_system_attrs(self, trial_id: int) -> dict[str, Any]:
         """Read the optuna-internal attributes of a trial.
 
         Args:
