@@ -1,9 +1,8 @@
+from __future__ import annotations
+
 import enum
 import math
 from typing import Any
-from typing import List
-from typing import Optional
-from typing import Tuple
 
 from sqlalchemy import asc
 from sqlalchemy import case
@@ -37,8 +36,7 @@ try:
     _Column = mapped_column
 except ImportError:
     # TODO(Shinichi): Remove this after dropping support for SQLAlchemy<2.0.
-    from sqlalchemy import Column as _Column  # type: ignore[assignment]
-
+    from sqlalchemy import Column as _Column  # type: ignore[assignment, no-redef]
 
 # Don't modify this version number anymore.
 # The schema management functionality has been moved to alembic.
@@ -77,7 +75,7 @@ class StudyModel(BaseModel):
         return study
 
     @classmethod
-    def find_by_name(cls, study_name: str, session: orm.Session) -> Optional["StudyModel"]:
+    def find_by_name(cls, study_name: str, session: orm.Session) -> "StudyModel" | None:
         study = session.query(cls).filter(cls.study_name == study_name).one_or_none()
 
         return study
@@ -104,7 +102,7 @@ class StudyDirectionModel(BaseModel):
     )
 
     @classmethod
-    def where_study_id(cls, study_id: int, session: orm.Session) -> List["StudyDirectionModel"]:
+    def where_study_id(cls, study_id: int, session: orm.Session) -> list["StudyDirectionModel"]:
         return session.query(cls).filter(cls.study_id == study_id).all()
 
 
@@ -123,7 +121,7 @@ class StudyUserAttributeModel(BaseModel):
     @classmethod
     def find_by_study_and_key(
         cls, study: StudyModel, key: str, session: orm.Session
-    ) -> Optional["StudyUserAttributeModel"]:
+    ) -> "StudyUserAttributeModel" | None:
         attribute = (
             session.query(cls)
             .filter(cls.study_id == study.study_id)
@@ -136,7 +134,7 @@ class StudyUserAttributeModel(BaseModel):
     @classmethod
     def where_study_id(
         cls, study_id: int, session: orm.Session
-    ) -> List["StudyUserAttributeModel"]:
+    ) -> list["StudyUserAttributeModel"]:
         return session.query(cls).filter(cls.study_id == study_id).all()
 
 
@@ -155,7 +153,7 @@ class StudySystemAttributeModel(BaseModel):
     @classmethod
     def find_by_study_and_key(
         cls, study: StudyModel, key: str, session: orm.Session
-    ) -> Optional["StudySystemAttributeModel"]:
+    ) -> "StudySystemAttributeModel" | None:
         attribute = (
             session.query(cls)
             .filter(cls.study_id == study.study_id)
@@ -168,7 +166,7 @@ class StudySystemAttributeModel(BaseModel):
     @classmethod
     def where_study_id(
         cls, study_id: int, session: orm.Session
-    ) -> List["StudySystemAttributeModel"]:
+    ) -> list["StudySystemAttributeModel"]:
         return session.query(cls).filter(cls.study_id == study_id).all()
 
 
@@ -189,11 +187,10 @@ class TrialModel(BaseModel):
     )
 
     @classmethod
-    def find_max_value_trial(
-        cls, study_id: int, objective: int, session: orm.Session
-    ) -> "TrialModel":
+    def find_max_value_trial_id(cls, study_id: int, objective: int, session: orm.Session) -> int:
         trial = (
             session.query(cls)
+            .with_entities(cls.trial_id)
             .filter(cls.study_id == study_id)
             .filter(cls.state == TrialState.COMPLETE)
             .join(TrialValueModel)
@@ -212,14 +209,13 @@ class TrialModel(BaseModel):
         )
         if trial is None:
             raise ValueError(NOT_FOUND_MSG)
-        return trial
+        return trial[0]
 
     @classmethod
-    def find_min_value_trial(
-        cls, study_id: int, objective: int, session: orm.Session
-    ) -> "TrialModel":
+    def find_min_value_trial_id(cls, study_id: int, objective: int, session: orm.Session) -> int:
         trial = (
             session.query(cls)
+            .with_entities(cls.trial_id)
             .filter(cls.study_id == study_id)
             .filter(cls.state == TrialState.COMPLETE)
             .join(TrialValueModel)
@@ -238,7 +234,7 @@ class TrialModel(BaseModel):
         )
         if trial is None:
             raise ValueError(NOT_FOUND_MSG)
-        return trial
+        return trial[0]
 
     @classmethod
     def find_or_raise_by_id(
@@ -259,10 +255,7 @@ class TrialModel(BaseModel):
 
     @classmethod
     def count(
-        cls,
-        session: orm.Session,
-        study: Optional[StudyModel] = None,
-        state: Optional[TrialState] = None,
+        cls, session: orm.Session, study: StudyModel | None = None, state: TrialState | None = None
     ) -> int:
         trial_count = session.query(func.count(cls.trial_id))
         if study is not None:
@@ -294,7 +287,7 @@ class TrialUserAttributeModel(BaseModel):
     @classmethod
     def find_by_trial_and_key(
         cls, trial: TrialModel, key: str, session: orm.Session
-    ) -> Optional["TrialUserAttributeModel"]:
+    ) -> "TrialUserAttributeModel" | None:
         attribute = (
             session.query(cls)
             .filter(cls.trial_id == trial.trial_id)
@@ -307,7 +300,7 @@ class TrialUserAttributeModel(BaseModel):
     @classmethod
     def where_trial_id(
         cls, trial_id: int, session: orm.Session
-    ) -> List["TrialUserAttributeModel"]:
+    ) -> list["TrialUserAttributeModel"]:
         return session.query(cls).filter(cls.trial_id == trial_id).all()
 
 
@@ -326,7 +319,7 @@ class TrialSystemAttributeModel(BaseModel):
     @classmethod
     def find_by_trial_and_key(
         cls, trial: TrialModel, key: str, session: orm.Session
-    ) -> Optional["TrialSystemAttributeModel"]:
+    ) -> "TrialSystemAttributeModel" | None:
         attribute = (
             session.query(cls)
             .filter(cls.trial_id == trial.trial_id)
@@ -339,7 +332,7 @@ class TrialSystemAttributeModel(BaseModel):
     @classmethod
     def where_trial_id(
         cls, trial_id: int, session: orm.Session
-    ) -> List["TrialSystemAttributeModel"]:
+    ) -> list["TrialSystemAttributeModel"]:
         return session.query(cls).filter(cls.trial_id == trial_id).all()
 
 
@@ -356,19 +349,17 @@ class TrialParamModel(BaseModel):
         TrialModel, backref=orm.backref("params", cascade="all, delete-orphan")
     )
 
-    def check_and_add(self, session: orm.Session) -> None:
-        self._check_compatibility_with_previous_trial_param_distributions(session)
+    def check_and_add(self, session: orm.Session, study_id: int) -> None:
+        self._check_compatibility_with_previous_trial_param_distributions(session, study_id)
         session.add(self)
 
     def _check_compatibility_with_previous_trial_param_distributions(
-        self, session: orm.Session
+        self, session: orm.Session, study_id: int
     ) -> None:
-        trial = TrialModel.find_or_raise_by_id(self.trial_id, session)
-
         previous_record = (
             session.query(TrialParamModel)
             .join(TrialModel)
-            .filter(TrialModel.study_id == trial.study_id)
+            .filter(TrialModel.study_id == study_id)
             .filter(TrialParamModel.param_name == self.param_name)
             .first()
         )
@@ -381,7 +372,7 @@ class TrialParamModel(BaseModel):
     @classmethod
     def find_by_trial_and_param_name(
         cls, trial: TrialModel, param_name: str, session: orm.Session
-    ) -> Optional["TrialParamModel"]:
+    ) -> "TrialParamModel" | None:
         param_distribution = (
             session.query(cls)
             .filter(cls.trial_id == trial.trial_id)
@@ -403,7 +394,7 @@ class TrialParamModel(BaseModel):
         return param_distribution
 
     @classmethod
-    def where_trial_id(cls, trial_id: int, session: orm.Session) -> List["TrialParamModel"]:
+    def where_trial_id(cls, trial_id: int, session: orm.Session) -> list["TrialParamModel"]:
         trial_params = session.query(cls).filter(cls.trial_id == trial_id).all()
 
         return trial_params
@@ -428,10 +419,7 @@ class TrialValueModel(BaseModel):
     )
 
     @classmethod
-    def value_to_stored_repr(
-        cls,
-        value: float,
-    ) -> Tuple[Optional[float], TrialValueType]:
+    def value_to_stored_repr(cls, value: float) -> tuple[float | None, TrialValueType]:
         if value == float("inf"):
             return (None, cls.TrialValueType.INF_POS)
         elif value == float("-inf"):
@@ -440,7 +428,7 @@ class TrialValueModel(BaseModel):
             return (value, cls.TrialValueType.FINITE)
 
     @classmethod
-    def stored_repr_to_value(cls, value: Optional[float], float_type: TrialValueType) -> float:
+    def stored_repr_to_value(cls, value: float | None, float_type: TrialValueType) -> float:
         if float_type == cls.TrialValueType.INF_POS:
             assert value is None
             return float("inf")
@@ -455,7 +443,7 @@ class TrialValueModel(BaseModel):
     @classmethod
     def find_by_trial_and_objective(
         cls, trial: TrialModel, objective: int, session: orm.Session
-    ) -> Optional["TrialValueModel"]:
+    ) -> "TrialValueModel" | None:
         trial_value = (
             session.query(cls)
             .filter(cls.trial_id == trial.trial_id)
@@ -466,7 +454,7 @@ class TrialValueModel(BaseModel):
         return trial_value
 
     @classmethod
-    def where_trial_id(cls, trial_id: int, session: orm.Session) -> List["TrialValueModel"]:
+    def where_trial_id(cls, trial_id: int, session: orm.Session) -> list["TrialValueModel"]:
         trial_values = (
             session.query(cls).filter(cls.trial_id == trial_id).order_by(asc(cls.objective)).all()
         )
@@ -495,9 +483,8 @@ class TrialIntermediateValueModel(BaseModel):
 
     @classmethod
     def intermediate_value_to_stored_repr(
-        cls,
-        value: float,
-    ) -> Tuple[Optional[float], TrialIntermediateValueType]:
+        cls, value: float
+    ) -> tuple[float | None, TrialIntermediateValueType]:
         if math.isnan(value):
             return (None, cls.TrialIntermediateValueType.NAN)
         elif value == float("inf"):
@@ -509,7 +496,7 @@ class TrialIntermediateValueModel(BaseModel):
 
     @classmethod
     def stored_repr_to_intermediate_value(
-        cls, value: Optional[float], float_type: TrialIntermediateValueType
+        cls, value: float | None, float_type: TrialIntermediateValueType
     ) -> float:
         if float_type == cls.TrialIntermediateValueType.NAN:
             assert value is None
@@ -528,7 +515,7 @@ class TrialIntermediateValueModel(BaseModel):
     @classmethod
     def find_by_trial_and_step(
         cls, trial: TrialModel, step: int, session: orm.Session
-    ) -> Optional["TrialIntermediateValueModel"]:
+    ) -> "TrialIntermediateValueModel" | None:
         trial_intermediate_value = (
             session.query(cls)
             .filter(cls.trial_id == trial.trial_id)
@@ -541,7 +528,7 @@ class TrialIntermediateValueModel(BaseModel):
     @classmethod
     def where_trial_id(
         cls, trial_id: int, session: orm.Session
-    ) -> List["TrialIntermediateValueModel"]:
+    ) -> list["TrialIntermediateValueModel"]:
         trial_intermediate_values = session.query(cls).filter(cls.trial_id == trial_id).all()
 
         return trial_intermediate_values
@@ -560,9 +547,15 @@ class TrialHeartbeatModel(BaseModel):
 
     @classmethod
     def where_trial_id(
-        cls, trial_id: int, session: orm.Session
-    ) -> Optional["TrialHeartbeatModel"]:
-        return session.query(cls).filter(cls.trial_id == trial_id).one_or_none()
+        cls, trial_id: int, session: orm.Session, for_update: bool = False
+    ) -> "TrialHeartbeatModel" | None:
+
+        query = session.query(cls).filter(cls.trial_id == trial_id)
+
+        if for_update:
+            query = query.with_for_update()
+
+        return query.one_or_none()
 
 
 class VersionInfoModel(BaseModel):
@@ -574,6 +567,6 @@ class VersionInfoModel(BaseModel):
     library_version = _Column(String(MAX_VERSION_LENGTH))
 
     @classmethod
-    def find(cls, session: orm.Session) -> Optional["VersionInfoModel"]:
+    def find(cls, session: orm.Session) -> "VersionInfoModel" | None:
         version_info = session.query(cls).one_or_none()
         return version_info
