@@ -4,6 +4,7 @@ from optuna.study._study_summary import StudySummary
 from optuna.study.study import copy_study
 from optuna.study.study import create_study
 from optuna.study.study import delete_study
+from optuna.study.study import get_all_study_names
 from optuna.study.study import get_all_study_summaries
 from optuna.study.study import load_study
 from optuna.study.study import Study
@@ -16,6 +17,7 @@ __all__ = [
     "copy_study",
     "create_study",
     "delete_study",
+    "get_all_study_names",
     "get_all_study_summaries",
     "load_study",
     "Study",
