@@ -1,18 +1,25 @@
+from __future__ import annotations
+
 import copy
 import math
 from typing import Optional
 from typing import Sequence
+from typing import TYPE_CHECKING
 from typing import Union
 import warnings
 
 import optuna
 from optuna import logging
 from optuna import pruners
-from optuna import trial as trial_module
 from optuna.trial import FrozenTrial
 from optuna.trial import TrialState
 
 
+if TYPE_CHECKING:
+    from optuna import Study
+    from optuna import Trial
+
+
 # This is used for propagating warning message to Study.optimize.
 STUDY_TELL_WARNING_KEY = "STUDY_TELL_WARNING"
 
@@ -20,8 +27,8 @@ STUDY_TELL_WARNING_KEY = "STUDY_TELL_WARNING"
 _logger = logging.get_logger(__name__)
 
 
-def _get_frozen_trial(study: "optuna.Study", trial: Union[trial_module.Trial, int]) -> FrozenTrial:
-    if isinstance(trial, trial_module.Trial):
+def _get_frozen_trial(study: Study, trial: Union[Trial, int]) -> FrozenTrial:
+    if isinstance(trial, optuna.Trial):
         trial_id = trial._trial_id
     elif isinstance(trial, int):
         trial_number = trial
@@ -58,7 +65,7 @@ def _check_state_and_values(
         raise ValueError(f"Cannot tell with state {state}.")
 
 
-def _check_values_are_feasible(study: "optuna.Study", values: Sequence[float]) -> Optional[str]:
+def _check_values_are_feasible(study: Study, values: Sequence[float]) -> Optional[str]:
     for v in values:
         # TODO(Imamura): Construct error message taking into account all values and do not early
         # return `value` is assumed to be ignored on failure so we can set it to any value.
@@ -80,8 +87,8 @@ def _check_values_are_feasible(study: "optuna.Study", values: Sequence[float]) -
 
 
 def _tell_with_warning(
-    study: "optuna.Study",
-    trial: Union[trial_module.Trial, int],
+    study: Study,
+    trial: Union[Trial, int],
     value_or_values: Optional[Union[float, Sequence[float]]] = None,
     state: Optional[TrialState] = None,
     skip_if_finished: bool = False,
