@@ -3,9 +3,11 @@ from optuna.terminator.erroreval import BaseErrorEvaluator
 from optuna.terminator.erroreval import CrossValidationErrorEvaluator
 from optuna.terminator.erroreval import report_cross_validation_scores
 from optuna.terminator.erroreval import StaticErrorEvaluator
+from optuna.terminator.improvement.emmr import EMMREvaluator
 from optuna.terminator.improvement.evaluator import BaseImprovementEvaluator
 from optuna.terminator.improvement.evaluator import BestValueStagnationEvaluator
 from optuna.terminator.improvement.evaluator import RegretBoundEvaluator
+from optuna.terminator.median_erroreval import MedianErrorEvaluator
 from optuna.terminator.terminator import BaseTerminator
 from optuna.terminator.terminator import Terminator
 
@@ -16,9 +18,11 @@ __all__ = [
     "CrossValidationErrorEvaluator",
     "report_cross_validation_scores",
     "StaticErrorEvaluator",
+    "MedianErrorEvaluator",
     "BaseImprovementEvaluator",
     "BestValueStagnationEvaluator",
     "RegretBoundEvaluator",
+    "EMMREvaluator",
     "BaseTerminator",
     "Terminator",
 ]
