@@ -1,5 +1,6 @@
+from __future__ import annotations
+
 from typing import Any
-from typing import Dict
 
 from optuna.distributions import BaseDistribution
 
@@ -11,7 +12,7 @@ class UnsupportedDistribution(BaseDistribution):
     def _contains(self, param_value_in_internal_repr: float) -> bool:
         return True
 
-    def _asdict(self) -> Dict:
+    def _asdict(self) -> dict:
         return {}
 
     def to_internal_repr(self, param_value_in_external_repr: Any) -> float:
