@@ -9,6 +9,7 @@ from typing import NamedTuple
 
 import numpy as np
 
+from optuna.distributions import CategoricalDistribution
 from optuna.logging import get_logger
 from optuna.study import Study
 from optuna.trial import FrozenTrial
@@ -17,7 +18,6 @@ from optuna.visualization._plotly_imports import _imports
 from optuna.visualization._utils import _check_plot_args
 from optuna.visualization._utils import _filter_nonfinite
 from optuna.visualization._utils import _get_skipped_trial_numbers
-from optuna.visualization._utils import _is_categorical
 from optuna.visualization._utils import _is_log_scale
 from optuna.visualization._utils import _is_numerical
 from optuna.visualization._utils import _is_reverse_scale
@@ -58,28 +58,6 @@ def plot_parallel_coordinate(
 
     Note that, if a parameter contains missing values, a trial with missing values is not plotted.
 
-    Example:
-
-        The following code snippet shows how to plot the high-dimensional parameter relationships.
-
-        .. plotly::
-
-            import optuna
-
-
-            def objective(trial):
-                x = trial.suggest_float("x", -100, 100)
-                y = trial.suggest_categorical("y", [-1, 0, 1])
-                return x ** 2 + y
-
-
-            sampler = optuna.samplers.TPESampler(seed=10)
-            study = optuna.create_study(sampler=sampler)
-            study.optimize(objective, n_trials=10)
-
-            fig = optuna.visualization.plot_parallel_coordinate(study, params=["x", "y"])
-            fig.show()
-
     Args:
         study:
             A :class:`~optuna.study.Study` object whose trials are plotted for their target values.
@@ -203,13 +181,13 @@ def _get_parallel_coordinate_info(
     dims = []
     for dim_index, p_name in enumerate(sorted_params, start=1):
         values = []
+        is_categorical = False
         for t in trials:
             if t.number in skipped_trial_numbers:
                 continue
-
             if p_name in t.params:
                 values.append(t.params[p_name])
-
+                is_categorical |= isinstance(t.distributions[p_name], CategoricalDistribution)
         if _is_log_scale(trials, p_name):
             values = [math.log10(v) for v in values]
             min_value = min(values)
@@ -230,7 +208,7 @@ def _get_parallel_coordinate_info(
                 tickvals=tickvals,
                 ticktext=["{:.3g}".format(math.pow(10, x)) for x in tickvals],
             )
-        elif _is_categorical(trials, p_name):
+        elif is_categorical:
             vocab: defaultdict[int | str, int] = defaultdict(lambda: len(vocab))
 
             ticktext: list[str]
