@@ -6,9 +6,9 @@ from optuna._imports import try_import
 with try_import() as _imports:
     import plotly
     from plotly import __version__ as plotly_version
-    import plotly.graph_objs as go
-    from plotly.graph_objs import Contour
-    from plotly.graph_objs import Scatter
+    import plotly.graph_objects as go
+    from plotly.graph_objects import Contour
+    from plotly.graph_objects import Scatter
     from plotly.subplots import make_subplots
 
     if version.parse(plotly_version) < version.parse("4.0.0"):
