@@ -42,35 +42,6 @@ def plot_rank(
         :func:`~optuna.visualization.matplotlib.plot_rank` function would be different from
         those of the Plotly-based :func:`~optuna.visualization.plot_rank`.
 
-    Example:
-
-        The following code snippet shows how to plot the parameter relationship as a rank plot.
-
-        .. plot::
-
-            import optuna
-
-
-            def objective(trial):
-                x = trial.suggest_float("x", -100, 100)
-                y = trial.suggest_categorical("y", [-1, 0, 1])
-
-                c0 = 400 - (x + y)**2
-                trial.set_user_attr("constraint", [c0])
-
-                return x ** 2 + y
-
-
-            def constraints(trial):
-                return trial.user_attrs["constraint"]
-
-
-            sampler = optuna.samplers.TPESampler(seed=10, constraints_func=constraints)
-            study = optuna.create_study(sampler=sampler)
-            study.optimize(objective, n_trials=30)
-
-            optuna.visualization.matplotlib.plot_rank(study, params=["x", "y"])
-
     Args:
         study:
             A :class:`~optuna.study.Study` object whose trials are plotted for their target values.
@@ -160,4 +131,9 @@ def _add_rank_subplot(
     if info.yaxis.is_log:
         ax.set_yscale("log")
 
-    return ax.scatter(x=info.xs, y=info.ys, c=info.colors / 255, edgecolors="grey")
+    return ax.scatter(
+        x=[str(x) for x in info.xs] if info.xaxis.is_cat else info.xs,
+        y=[str(y) for y in info.ys] if info.yaxis.is_cat else info.ys,
+        c=info.colors / 255,
+        edgecolors="grey",
+    )
