@@ -4,18 +4,13 @@ import io
 from typing import TYPE_CHECKING
 
 import boto3
+from moto import mock_aws
 import pytest
 
 from optuna.artifacts import Boto3ArtifactStore
 from optuna.artifacts.exceptions import ArtifactNotFound
 
 
-try:
-    # TODO(nabenabe0928): Replace it with `from moto import mock_aws` after dropping Python3.7.
-    from moto import mock_aws
-except ImportError:
-    from moto import mock_s3 as mock_aws  # type: ignore[attr-defined,no-redef]
-
 if TYPE_CHECKING:
     from collections.abc import Iterator
 
