@@ -1,9 +1,8 @@
+from __future__ import annotations
+
+from collections.abc import Mapping
+from collections.abc import Sequence
 import itertools
-from typing import Dict
-from typing import List
-from typing import Mapping
-from typing import Sequence
-from typing import Union
 from typing import ValuesView
 
 import numpy as np
@@ -76,7 +75,7 @@ def test_study_optimize_with_exceeding_number_of_trials() -> None:
         return trial.suggest_int("a", 0, 100)
 
     # When `n_trials` is `None`, the optimization stops just after all grids are evaluated.
-    search_space: Dict[str, List[GridValueType]] = {"a": [0, 50]}
+    search_space: dict[str, list[GridValueType]] = {"a": [0, 50]}
     study = optuna.create_study(sampler=samplers.GridSampler(search_space))
     study.optimize(objective, n_trials=None)
     assert len(study.trials) == 2
@@ -88,7 +87,7 @@ def test_study_optimize_with_exceeding_number_of_trials() -> None:
 
 def test_study_optimize_with_pruning() -> None:
     # Pruned trials should count towards grid consumption.
-    search_space: Dict[str, List[GridValueType]] = {"a": [0, 50]}
+    search_space: dict[str, list[GridValueType]] = {"a": [0, 50]}
     study = optuna.create_study(sampler=samplers.GridSampler(search_space))
     study.optimize(pruned_objective, n_trials=None)
     assert len(study.trials) == 2
@@ -99,7 +98,7 @@ def test_study_optimize_with_fail() -> None:
         return trial.suggest_int("a", 0, 100)
 
     # Failed trials should count towards grid consumption.
-    search_space: Dict[str, List[GridValueType]] = {"a": [0, 50]}
+    search_space: dict[str, list[GridValueType]] = {"a": [0, 50]}
     study = optuna.create_study(sampler=samplers.GridSampler(search_space))
     study.optimize(fail_objective, n_trials=1, catch=ValueError)
     study.optimize(objective, n_trials=None)
@@ -184,7 +183,7 @@ def test_cast_value() -> None:
 
 
 def test_has_same_search_space() -> None:
-    search_space: Dict[str, List[Union[int, str]]] = {"x": [3, 2, 1], "y": ["a", "b", "c"]}
+    search_space: dict[str, list[int | str]] = {"x": [3, 2, 1], "y": ["a", "b", "c"]}
     sampler = samplers.GridSampler(search_space)
     assert sampler._same_search_space(search_space)
     assert sampler._same_search_space({"x": [3, 2, 1], "y": ["a", "b", "c"]})
@@ -254,3 +253,12 @@ def test_nan() -> None:
         lambda trial: 1 if np.isnan(trial.suggest_categorical("x", [0, float("nan")])) else 0
     )
     assert len(study.get_trials()) == 2
+
+
+def test_is_exhausted() -> None:
+    search_space = {"a": [0, 50]}
+    sampler = samplers.GridSampler(search_space)
+    study = optuna.create_study(sampler=sampler)
+    assert not sampler.is_exhausted(study)
+    study.optimize(lambda trial: trial.suggest_categorical("a", [0, 50]))
+    assert sampler.is_exhausted(study)
