@@ -21,8 +21,6 @@ from optuna.distributions import CategoricalChoiceType
 from optuna.distributions import CategoricalDistribution
 from optuna.distributions import FloatDistribution
 from optuna.distributions import IntDistribution
-from optuna.integration.botorch import logei_candidates_func
-from optuna.integration.botorch import qei_candidates_func
 from optuna.samplers import BaseSampler
 from optuna.samplers._lazy_random_state import LazyRandomState
 from optuna.study import Study
@@ -33,6 +31,16 @@ from optuna.trial import Trial
 from optuna.trial import TrialState
 
 
+def get_gp_sampler(
+    *, n_startup_trials: int = 0, deterministic_objective: bool = False, seed: int | None = None
+) -> optuna.samplers.GPSampler:
+    return optuna.samplers.GPSampler(
+        n_startup_trials=n_startup_trials,
+        seed=seed,
+        deterministic_objective=deterministic_objective,
+    )
+
+
 parametrize_sampler = pytest.mark.parametrize(
     "sampler_class",
     [
@@ -41,27 +49,11 @@ parametrize_sampler = pytest.mark.parametrize(
         lambda: optuna.samplers.TPESampler(n_startup_trials=0, multivariate=True),
         lambda: optuna.samplers.CmaEsSampler(n_startup_trials=0),
         lambda: optuna.samplers.CmaEsSampler(n_startup_trials=0, use_separable_cma=True),
-        pytest.param(
-            lambda: optuna.integration.PyCmaSampler(n_startup_trials=0),
-            marks=pytest.mark.integration,
-        ),
         optuna.samplers.NSGAIISampler,
         optuna.samplers.NSGAIIISampler,
         optuna.samplers.QMCSampler,
-        pytest.param(
-            lambda: optuna.integration.BoTorchSampler(
-                n_startup_trials=0,
-                candidates_func=logei_candidates_func,
-            ),
-            marks=pytest.mark.integration,
-        ),
-        pytest.param(
-            lambda: optuna.integration.BoTorchSampler(
-                n_startup_trials=0,
-                candidates_func=qei_candidates_func,
-            ),
-            marks=pytest.mark.integration,
-        ),
+        lambda: get_gp_sampler(n_startup_trials=0),
+        lambda: get_gp_sampler(n_startup_trials=0, deterministic_objective=True),
     ],
 )
 parametrize_relative_sampler = pytest.mark.parametrize(
@@ -70,10 +62,8 @@ parametrize_relative_sampler = pytest.mark.parametrize(
         lambda: optuna.samplers.TPESampler(n_startup_trials=0, multivariate=True),
         lambda: optuna.samplers.CmaEsSampler(n_startup_trials=0),
         lambda: optuna.samplers.CmaEsSampler(n_startup_trials=0, use_separable_cma=True),
-        pytest.param(
-            lambda: optuna.integration.PyCmaSampler(n_startup_trials=0),
-            marks=pytest.mark.integration,
-        ),
+        lambda: get_gp_sampler(n_startup_trials=0),
+        lambda: get_gp_sampler(n_startup_trials=0, deterministic_objective=True),
     ],
 )
 parametrize_multi_objective_sampler = pytest.mark.parametrize(
@@ -82,43 +72,30 @@ parametrize_multi_objective_sampler = pytest.mark.parametrize(
         optuna.samplers.NSGAIISampler,
         optuna.samplers.NSGAIIISampler,
         lambda: optuna.samplers.TPESampler(n_startup_trials=0),
-        pytest.param(
-            lambda: optuna.integration.BoTorchSampler(n_startup_trials=0),
-            marks=pytest.mark.integration,
-        ),
     ],
 )
-sampler_class_with_seed: dict[str, tuple[Callable[[int], BaseSampler], bool]] = {
-    "RandomSampler": (lambda seed: optuna.samplers.RandomSampler(seed=seed), False),
-    "TPESampler": (lambda seed: optuna.samplers.TPESampler(seed=seed), False),
-    "multivariate TPESampler": (
-        lambda seed: optuna.samplers.TPESampler(multivariate=True, seed=seed),
-        False,
+
+
+sampler_class_with_seed: dict[str, Callable[[int], BaseSampler]] = {
+    "RandomSampler": lambda seed: optuna.samplers.RandomSampler(seed=seed),
+    "TPESampler": lambda seed: optuna.samplers.TPESampler(seed=seed),
+    "multivariate TPESampler": lambda seed: optuna.samplers.TPESampler(
+        multivariate=True, seed=seed
     ),
-    "CmaEsSampler": (lambda seed: optuna.samplers.CmaEsSampler(seed=seed), False),
-    "separable CmaEsSampler": (
-        lambda seed: optuna.samplers.CmaEsSampler(seed=seed, use_separable_cma=True),
-        False,
+    "CmaEsSampler": lambda seed: optuna.samplers.CmaEsSampler(seed=seed),
+    "separable CmaEsSampler": lambda seed: optuna.samplers.CmaEsSampler(
+        seed=seed, use_separable_cma=True
     ),
-    "PyCmaSampler": (lambda seed: optuna.integration.PyCmaSampler(seed=seed), True),
-    "NSGAIISampler": (lambda seed: optuna.samplers.NSGAIISampler(seed=seed), False),
-    "NSGAIIISampler": (lambda seed: optuna.samplers.NSGAIIISampler(seed=seed), False),
-    "QMCSampler": (lambda seed: optuna.samplers.QMCSampler(seed=seed), False),
-    "BoTorchSampler": (lambda seed: optuna.integration.BoTorchSampler(seed=seed), True),
+    "NSGAIISampler": lambda seed: optuna.samplers.NSGAIISampler(seed=seed),
+    "NSGAIIISampler": lambda seed: optuna.samplers.NSGAIIISampler(seed=seed),
+    "QMCSampler": lambda seed: optuna.samplers.QMCSampler(seed=seed),
+    "GPSampler": lambda seed: get_gp_sampler(seed=seed, n_startup_trials=0),
 }
 param_sampler_with_seed = []
 param_sampler_name_with_seed = []
-for sampler_name, (sampler_class, integration_flag) in sampler_class_with_seed.items():
-    if integration_flag:
-        param_sampler_with_seed.append(
-            pytest.param(sampler_class, id=sampler_name, marks=pytest.mark.integration)
-        )
-        param_sampler_name_with_seed.append(
-            pytest.param(sampler_name, marks=pytest.mark.integration)
-        )
-    else:
-        param_sampler_with_seed.append(pytest.param(sampler_class, id=sampler_name))
-        param_sampler_name_with_seed.append(pytest.param(sampler_name))
+for sampler_name, sampler_class in sampler_class_with_seed.items():
+    param_sampler_with_seed.append(pytest.param(sampler_class, id=sampler_name))
+    param_sampler_name_with_seed.append(pytest.param(sampler_name))
 parametrize_sampler_with_seed = pytest.mark.parametrize("sampler_class", param_sampler_with_seed)
 parametrize_sampler_name_with_seed = pytest.mark.parametrize(
     "sampler_name", param_sampler_name_with_seed
@@ -132,12 +109,6 @@ parametrize_sampler_name_with_seed = pytest.mark.parametrize(
         (lambda: optuna.samplers.TPESampler(n_startup_trials=0), True, True),
         (lambda: optuna.samplers.TPESampler(n_startup_trials=0, multivariate=True), True, True),
         (lambda: optuna.samplers.CmaEsSampler(n_startup_trials=0), True, True),
-        pytest.param(
-            lambda: optuna.integration.PyCmaSampler(n_startup_trials=0),
-            False,
-            True,
-            marks=pytest.mark.integration,
-        ),
         (optuna.samplers.NSGAIISampler, True, True),
         (optuna.samplers.NSGAIIISampler, True, True),
         (
@@ -149,12 +120,7 @@ parametrize_sampler_name_with_seed = pytest.mark.parametrize(
         ),
         (lambda: optuna.samplers.GridSampler(search_space={"x": [0]}), True, False),
         (lambda: optuna.samplers.QMCSampler(), False, True),
-        pytest.param(
-            lambda: optuna.integration.BoTorchSampler(n_startup_trials=0),
-            False,
-            True,
-            marks=pytest.mark.integration,
-        ),
+        (lambda: get_gp_sampler(n_startup_trials=0), True, True),
     ],
 )
 def test_sampler_reseed_rng(
@@ -226,10 +192,6 @@ def parametrize_suggest_method(name: str) -> MarkDecorator:
     "sampler_class",
     [
         lambda: optuna.samplers.CmaEsSampler(n_startup_trials=0),
-        pytest.param(
-            lambda: optuna.integration.PyCmaSampler(n_startup_trials=0),
-            marks=pytest.mark.integration,
-        ),
     ],
 )
 def test_raise_error_for_samplers_during_multi_objectives(
@@ -1029,7 +991,7 @@ def run_optimize(
         return a + b + c + d + e + f + g
 
     hash_dict[k] = hash("nondeterministic hash")
-    sampler = sampler_class_with_seed[sampler_name][0](1)
+    sampler = sampler_class_with_seed[sampler_name](1)
     study = optuna.create_study(sampler=sampler)
     study.optimize(objective, n_trials=15)
     sequence_dict[k] = list(study.trials[-1].params.values())
@@ -1094,7 +1056,6 @@ def test_cache_is_invalidated(
         trial.suggest_float("x", -10, 10)
         trial.suggest_float("y", -10, 10)
         assert trial._relative_params is not None
-        assert study._thread_local.cached_all_trials is not None
         return -1
 
     study.optimize(objective, n_trials=10, n_jobs=n_jobs)
