@@ -486,11 +486,6 @@ def test_set_trial_param(storage_mode: str) -> None:
         # Check set_param breaks neither get_trial nor get_trial_params.
         assert storage.get_trial(trial_id_1).params == {"x": 0.5, "y": "Meguro"}
         assert storage.get_trial_params(trial_id_1) == {"x": 0.5, "y": "Meguro"}
-        # Duplicated registration should overwrite.
-        storage.set_trial_param(trial_id_1, "x", 0.6, distribution_x)
-        assert storage.get_trial_param(trial_id_1, "x") == 0.6
-        assert storage.get_trial(trial_id_1).params == {"x": 0.6, "y": "Meguro"}
-        assert storage.get_trial_params(trial_id_1) == {"x": 0.6, "y": "Meguro"}
 
         # Set params to another trial.
         storage.set_trial_param(trial_id_2, "x", 0.3, distribution_x)
@@ -951,7 +946,36 @@ def test_get_best_trial(storage_mode: str, direction: StudyDirection, values: Li
         assert storage.get_best_trial(study_id).value == expected_value
 
 
-def test_get_trials_excluded_trial_ids() -> None:
+def test_get_trials_included_trial_ids() -> None:
+    storage_mode = "sqlite"
+
+    with StorageSupplier(storage_mode) as storage:
+        assert isinstance(storage, RDBStorage)
+        study_id = storage.create_new_study(directions=[StudyDirection.MINIMIZE])
+
+        trial_id = storage.create_new_trial(study_id)
+        trial_id_greater_than = trial_id + 500000
+
+        trials = storage._get_trials(
+            study_id,
+            states=None,
+            included_trial_ids=set(),
+            trial_id_greater_than=trial_id_greater_than,
+        )
+        assert len(trials) == 0
+
+        # A large exclusion list used to raise errors. Check that it is not an issue.
+        # See https://github.com/optuna/optuna/issues/1457.
+        trials = storage._get_trials(
+            study_id,
+            states=None,
+            included_trial_ids=set(range(500000)),
+            trial_id_greater_than=trial_id_greater_than,
+        )
+        assert len(trials) == 1
+
+
+def test_get_trials_trial_id_greater_than() -> None:
     storage_mode = "sqlite"
 
     with StorageSupplier(storage_mode) as storage:
@@ -960,12 +984,14 @@ def test_get_trials_excluded_trial_ids() -> None:
 
         storage.create_new_trial(study_id)
 
-        trials = storage._get_trials(study_id, states=None, excluded_trial_ids=set())
+        trials = storage._get_trials(
+            study_id, states=None, included_trial_ids=set(), trial_id_greater_than=-1
+        )
         assert len(trials) == 1
 
-        # A large exclusion list used to raise errors. Check that it is not an issue.
-        # See https://github.com/optuna/optuna/issues/1457.
-        trials = storage._get_trials(study_id, states=None, excluded_trial_ids=set(range(500000)))
+        trials = storage._get_trials(
+            study_id, states=None, included_trial_ids=set(), trial_id_greater_than=500001
+        )
         assert len(trials) == 0
 
 
