@@ -7,7 +7,7 @@ Multi-objective Optimization with Optuna
 This tutorial showcases Optuna's multi-objective optimization feature by
 optimizing the validation accuracy of Fashion MNIST dataset and the FLOPS of the model implemented in PyTorch.
 
-We use `fvcore <https://github.com/facebookresearch/fvcore>`_ to measure FLOPS.
+We use `fvcore <https://github.com/facebookresearch/fvcore>`__ to measure FLOPS.
 """
 
 import torch
@@ -119,6 +119,15 @@ print("Number of finished trials: ", len(study.trials))
 
 
 ###################################################################################################
+# Note that the following sections requires the installation of `Plotly <https://plotly.com/python>`__ for visualization
+# and `scikit-learn <https://scikit-learn.org/stable>`__ for hyperparameter importance calculation:
+#
+# .. code-block:: console
+#
+#     $ pip install plotly
+#     $ pip install scikit-learn
+#     $ pip install nbformat  # Required if you are running this tutorial in Jupyter Notebook.
+#
 # Check trials on Pareto front visually.
 optuna.visualization.plot_pareto_front(study, target_names=["FLOPS", "accuracy"])
 
@@ -131,7 +140,7 @@ optuna.visualization.plot_pareto_front(study, target_names=["FLOPS", "accuracy"]
 print(f"Number of trials on the Pareto front: {len(study.best_trials)}")
 
 trial_with_highest_accuracy = max(study.best_trials, key=lambda t: t.values[1])
-print(f"Trial with highest accuracy: ")
+print("Trial with highest accuracy: ")
 print(f"\tnumber: {trial_with_highest_accuracy.number}")
 print(f"\tparams: {trial_with_highest_accuracy.params}")
 print(f"\tvalues: {trial_with_highest_accuracy.values}")
